/*
  Spins two addressable rings in opposite directions.

  Example ready to use with ESP-8266.
  To use with other boards, replace pin numbers accordingly.

  Before uploading, replace <YOUR WIFI NAME> and <YOUR WIFI PASSWORD>
  with your WiFi credentials.
*/

#include <Adafruit_NeoPixel.h>
#include <WiFiGateDevice.h>

Adafruit_NeoPixel ring1(16, D2, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel ring2(12, D3, NEO_GRB + NEO_KHZ800);

// frame rate: 20fps => 50ms interval between frames
const int FRAME_INTERVAL = 50;
const double MIN_INCREMENT = 0.005;
const double MAX_INCREMENT = 0.05;

WiFiGateDevice device("<YOUR WIFI NAME>", "<YOUR WIFI PASSWORD>");
GateFloat positionIncrement = device.factory.createFloat("input");
GateInt brightness = device.factory.createInt("input");

unsigned long nextFrame = 0;
double ring1Position = 0;
int ring1ToothCount = 3;
double ring2Position = 0.5;
int ring2ToothCount = 2;

void setup() {
  device.setName("LED controller");

  positionIncrement.setName("Speed");
  positionIncrement.setMin(MIN_INCREMENT);
  positionIncrement.setMax(MAX_INCREMENT);

  brightness.setName("Brightness");
  brightness.setMin(1);
  brightness.setMax(100);
  brightness.setValue(50);

  device.start();

  ring1.begin();
  ring1.show();
  ring2.begin();
  ring2.show();
}

void loop() {
  loopRings();
  device.loop();
}

// displays subsequent frames at given interval
void loopRings() {
  if (millis() > nextFrame) {
    nextFrame = millis() + FRAME_INTERVAL;
    movePosition();
    showFrame();
  }
}

// moves "center" of gradient in reciprocating motion
void movePosition() {
  ring1Position += positionIncrement.getValue();
  ring2Position -= positionIncrement.getValue() * ring1ToothCount / ring2ToothCount;
  if (ring1Position >= 1) {
    ring1Position -= 1;
  }
  if (ring2Position <= 0) {
    ring2Position += 1;
  }
}

// sets colors of individual pixels basing on current marker position
void showFrame() {
  for(int pixel = 0; pixel < ring1.numPixels(); pixel++) {
    double pixelPosition = (double) pixel / (ring1.numPixels());
    double colorFactor = getColorFactor(pixelPosition, ring1Position, ring1ToothCount);
    int color = (int)(colorFactor * brightness.getValue());
    ring1.setPixelColor(pixel, ring1.Color(0, color, 0));
  }
  ring1.show();

  for(int pixel = 0; pixel < ring2.numPixels(); pixel++) {
    double pixelPosition = (double) pixel / (ring2.numPixels());
    double colorFactor = getColorFactor(pixelPosition, ring2Position, ring2ToothCount);
    int color = (int)(colorFactor * brightness.getValue());
    ring2.setPixelColor(pixel, ring2.Color(color, 0, 0));
  }
  ring2.show();
}

// calculates factor for color calculation (extracted for clarity)
double getColorFactor(double pixelPosition, double ringPosition, int tooths) {
  double fullAngle = TWO_PI * tooths;
  double position = ringPosition - pixelPosition;
  double factor = (sin(fullAngle * position) + 0.5)/1.5;
  if (factor < 0) {
    return 0;
  }
  return factor;
}
